import 'package:flumponent/enum/enum.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class FlumLoader extends StatefulWidget {
  const FlumLoader(
      {Key key,
      this.loaderColorOne = Colors.redAccent,
      this.loaderColorTwo = Colors.green,
      this.loaderColorCenter = Colors.blueAccent,
      this.duration = const Duration(milliseconds: 1000),
      this.type = FlumLoaderType.android,
      this.loaderIconOne,
      this.loaderIconTwo,
      this.loaderIconThree,
      this.androidLoaderColor,
      this.loaderstrokeWidth = 4.0,
      this.size = FlumSize.MEDIUM,
      this.child})
      : super(key: key);

  /// Type of [Widget] used only in custom type and it is prominent over the loaderIconOne, loaderIconTwo, loaderIconThree in custom type
  final Widget child;

  /// Type of GFColor or [Color] which defines the color of the first from center dot in only  circle or square type of loader
  final Color loaderColorOne;

  /// Type of GFColor or [Color] which defines the color of the second from center dot in only  circle or square type of loader
  final Color loaderColorTwo;

  /// Type of GFColor or [Color] which defines the color of the center dot in only  circle or square type of loader
  final Color loaderColorCenter;

  /// Type of duration which defines the animation duration of the loader only in circle and square type of loader
  final Duration duration;

  /// Type of [FlumLoaderType] ie, android, ios, circle , square and custom
  final FlumLoaderType type;

  /// Type of [Widget] which takes text, icons or images for first dot only in custom type of loader
  final Widget loaderIconOne;

  /// Type of [Widget] which takes text, icons or images for second dot only in custom type of loader
  final Widget loaderIconTwo;

  /// Type of [Widget] which takes text, icons or images for third dot only in custom type of loader
  final Widget loaderIconThree;

  /// type of Animation<Color> used to change the color of the android loader only
  final Animation<Color> androidLoaderColor;

  /// type of [double] used to change the stroke width of the android loader only
  final double loaderstrokeWidth;

  /// type of [double] or [FlumSize] ie, small , medium or large which is used
  /// to change the size of android, ios, circle and square loaders only
  final double size;

  @override
  _FlumLoaderState createState() => _FlumLoaderState();
}

class _FlumLoaderState extends State<FlumLoader>
    with SingleTickerProviderStateMixin {
  Animation<double> loaderAnimation1;
  Animation<double> loaderAnimation2;
  Animation<double> loaderAnimation3;
  Animation<double> loaderAnimation4;
  Animation<double> loaderAnimation5;
  AnimationController controller;

  @override
  void initState() {
    super.initState();

    controller = AnimationController(duration: widget.duration, vsync: this);

    loaderAnimation1 = Tween<double>(begin: 0, end: 1).animate(
      CurvedAnimation(
        parent: controller,
        curve: const Interval(
          0.2,
          0.91,
          curve: Curves.linear,
        ),
      ),
    );

    loaderAnimation2 = Tween<double>(begin: 0, end: 1).animate(
      CurvedAnimation(
        parent: controller,
        curve: const Interval(
          0.1,
          0.81,
          curve: Curves.linear,
        ),
      ),
    );

    loaderAnimation3 = Tween<double>(begin: 0, end: 1).animate(
      CurvedAnimation(
        parent: controller,
        curve: const Interval(
          0,
          0.71,
          curve: Curves.linear,
        ),
      ),
    );

    loaderAnimation4 = Tween<double>(begin: 0, end: 1).animate(
      CurvedAnimation(
        parent: controller,
        curve: const Interval(
          0.1,
          0.81,
          curve: Curves.linear,
        ),
      ),
    );

    loaderAnimation5 = Tween<double>(begin: 0, end: 1).animate(
      CurvedAnimation(
        parent: controller,
        curve: const Interval(
          0.2,
          0.91,
          curve: Curves.linear,
        ),
      ),
    );

    controller.addListener(() {
      setState(() {});
    });

    controller.repeat();
  }

  @override
  Widget build(BuildContext context) => Container(
        child: widget.child != null
            ? _Loader(
                radius: widget.size * 0.3,
                type: widget.type,
                child: widget.child,
              )
            : widget.type == FlumLoaderType.android
                ? Center(
                    child: Container(
                      height: widget.size * 0.7,
                      width: widget.size * 0.7,
                      child: CircularProgressIndicator(
                        valueColor: widget.androidLoaderColor,
                        strokeWidth: widget.loaderstrokeWidth,
//              value: 20,
                      ),
                    ),
                  )
                : widget.type == FlumLoaderType.ios
                    ? Center(
                        child: CupertinoActivityIndicator(
                          radius: widget.size * 0.4,
                        ),
                      )
                    : Row(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: <Widget>[
                          Opacity(
                            opacity: loaderAnimation1.value <= 0.3
                                ? 2.5 * loaderAnimation1.value
                                : (loaderAnimation1.value > 0.30 &&
                                        loaderAnimation1.value <= 0.70)
                                    ? 1.0
                                    : 2.5 - (2.5 * loaderAnimation1.value),
                            child: Padding(
                              padding: const EdgeInsets.only(right: 8),
                              child: _Loader(
                                radius: widget.size * 0.3,
                                color: widget.loaderColorOne,
                                type: widget.type,
                                icon: widget.loaderIconOne,
                                child: widget.child,
                              ),
                            ),
                          ),
                          Opacity(
                            opacity: loaderAnimation2.value <= 0.3
                                ? 2.5 * loaderAnimation2.value
                                : (loaderAnimation2.value > 0.30 &&
                                        loaderAnimation2.value <= 0.70)
                                    ? 1.0
                                    : 2.5 - (2.5 * loaderAnimation2.value),
                            child: Padding(
                              padding: const EdgeInsets.only(right: 8),
                              child: _Loader(
                                radius: widget.size * 0.44,
                                color: widget.loaderColorTwo,
                                type: widget.type,
                                icon: widget.loaderIconTwo,
                              ),
                            ),
                          ),
                          Opacity(
                            opacity: loaderAnimation3.value <= 0.3
                                ? 2.5 * loaderAnimation3.value
                                : (loaderAnimation3.value > 0.30 &&
                                        loaderAnimation3.value <= 0.70)
                                    ? 1.0
                                    : 2.5 - (2.5 * loaderAnimation3.value),
                            child: Padding(
                              padding: const EdgeInsets.only(right: 8),
                              child: _Loader(
                                radius: widget.size * 0.3,
                                color: widget.loaderColorCenter,
                                type: widget.type,
                                icon: widget.loaderIconThree,
                              ),
                            ),
                          ),
                        ],
                      ),
      );

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}

class _Loader extends StatelessWidget {
  const _Loader({
    Key key,
    this.radius,
    this.color,
    this.type,
    this.icon,
    this.size,
    this.child,
  }) : super(key: key);

  final double radius;
  final Color color;
  final FlumLoaderType type;
  final Widget icon;
  final double size;
  final Widget child;

  @override
  Widget build(BuildContext context) {
    BorderRadius _loaderRadius;
    BoxShape _loaderShape;
    double _width;
    double _height;

    if (type != FlumLoaderType.custom) {
      switch (type) {
        case FlumLoaderType.circle:
          _loaderShape = BoxShape.circle;
          _width = radius;
          _height = radius;
          break;
        case FlumLoaderType.oval:
          _loaderShape = BoxShape.rectangle;
          _width = radius * .8;
          _height = radius * 3;
          _loaderRadius = BorderRadius.all(Radius.elliptical(50, 150));
          break;
        case FlumLoaderType.square:
          _loaderShape = BoxShape.rectangle;
          _width = radius;
          _height = radius;
          break;
        case FlumLoaderType.bar:
        case FlumLoaderType.rectangle:
          _loaderShape = BoxShape.rectangle;
          _width = radius * .8;
          _height = radius * 3;
          break;
        case FlumLoaderType.capsule:
          _loaderShape = BoxShape.rectangle;
          _width = radius * .8;
          _height = radius * 3;
          _loaderRadius = BorderRadius.all(Radius.elliptical(100, 50));
          break;
        default:
          _loaderShape = BoxShape.circle;
          _width = radius;
          _height = radius;
      }
    }

    return Center(
      child: type == FlumLoaderType.custom
          ? Container(
        child: child != null ? child : icon ?? Container(),
      )
          : Container(
        width: _width,
        height: _height,
        decoration: BoxDecoration(
          color: color,
          shape: _loaderShape,
          borderRadius: _loaderRadius,
        ),
      ),
    );
  }
}
